var camera = {
  canvas: null,
  ctx: null,
  steps: [],
  halfStep: 0,
  dist: [],
  sectionCount: Game.projectionWidth / Game.sectWidth,
  cast: function () {
    this.ctx.clearRect(0,0,Game.projectionWidth,Game.projectionHeight);
    this.drawBackground();
    this.dist[this.sectionCount] = 0;
    for (let i = 0; i < this.sectionCount; i++) {
      this.dist[i] = this.trackRay(this.steps[i] + player.rotation, i);
      this.dist[this.sectionCount] = Math.max(this.dist[i], this.dist[this.sectionCount]);
    }
  },
  trackRay: function(angle, strip){
      angle = only360(angle);
      let isRight = angle > Game.twoPi * 0.75 || angle < Game.twoPi * 0.25;
      let isUp = angle > Math.PI;
      let distance = 0;
      let hit = {x: 0, y: 0};
      let playerX = player.x / Game.gameScale;
      let playerY = player.y / Game.gameScale;
      let wallType = 0; // future use
      let textureX = 0; // future use
      let mapWidth = Game.map.getWidth();
      let mapHeight = Game.map.getHeight();
      let slope = Math.sin(angle)/Math.cos(angle);
      let xChange = isRight ? 1 : -1; // xChange is in block steps
      let yChange = xChange * slope;
      // check left or right for ray collision with vertical wall
      let x = isRight ? Math.ceil(playerX) : Math.floor(playerX);
      let y = playerY + (x - playerX) * slope;
      while(x >= 0 && x < mapWidth && y >= 0 && y < mapHeight){
        let wallX = Math.floor(x + (isRight ? 0 : -1));
        let wallY = Math.floor(y);
        if(Game.map.getGrid(wallX, wallY)){ // we have a hit so calc the distance
          distance = Math.hypot(x * Game.gameScale - player.x, 
                                y * Game.gameScale - player.y);
          hit.x = x;
          hit.y = y;
          textureX = isRight? 1 - y % 1 : y % 1;
          wallType = Game.map.getGrid(wallX, wallY); // for later
          break;
        }
        x += xChange;
        y += yChange;
      }
      // now check up and down the grid
      slope = Math.cos(angle) / Math.sin(angle);
      yChange = isUp ? -1 : 1;
      xChange = yChange * slope;
      y = isUp ? Math.floor(playerY) : Math.ceil(playerY);
      x = playerX + (y - playerY) * slope;
      while(x >= 0 && x < mapWidth && y >= 0 && y < mapHeight){
        wallY = Math.floor(y + (isUp ? -1 : 0));
        wallX = Math.floor(x);
        if(Game.map.getGrid(wallX, wallY)){
          let newDistance = Math.hypot(x * Game.gameScale - player.x,
                                        y * Game.gameScale - player.y);
          if(!distance || newDistance < distance){
            distance = newDistance;
            hit.x = x;
            hit.y = y;
            textureX = isUp ? 1 - x % 1 : x % 1;
            wallType = Game.map.getGrid(wallX, wallY);
          }
          break;
        }
        y += yChange;
        x += xChange;
      }
      Game.map.drawRay(player, hit);
        // correct distance for fish eye distortion
        let noFish = distance * Math.cos(player.rotation - angle);
        this.renderStrip(strip, wallType, noFish, textureX);
      return distance;
  },
  drawBackground: function() {
    this.ctx.beginPath();
    this.ctx.fillStyle = Level.bgTopGrad;
    this.ctx.rect(0,0,this.canvas.width, this.canvas.height);
    this.ctx.closePath();
    this.ctx.fill();
    this.ctx.beginPath();
    this.ctx.fillStyle = Level.bgFloorGrad;
    this.ctx.rect(0,this.canvas.height/2,this.canvas.width, this.canvas.height);
    this.ctx.closePath();
    this.ctx.fill();    
  },
  renderStrip: function(strip, wallType, distance, textureX) {
    let height = Game.playerDistance / distance;
    let width = Math.round(Game.sectWidth * (1 / height)); //correct lateral distortion
    if(width < 1) {width = 1;}
    height = Math.round(height * Game.gameScale);
    textureX = Math.round(textureX * Game.gameScale);
    if (textureX > Game.gameScale - width) { //needs correcting
        textureX = Game.gameScale - width;
    }
    let top = Math.round((Game.projectionHeight - height) / 2);
    this.ctx.drawImage(Game.walls[wallType - 1], textureX, 0, width, Game.gameScale, strip * Game.sectWidth, top, Game.sectWidth, height);
  },
  init: function(canvId){
    this.canvas = document.getElementById(canvId);
    this.canvas.width = Game.projectionWidth;
    this.canvas.height = Game.projectionHeight;
    this.ctx = this.canvas.getContext("2d");
    let startPos = -this.sectionCount / 2;
    for (let i = 0; i < this.sectionCount; i++) {
      let projectionPosition = (startPos + i) * Game.sectWidth;
      let viewDistance = Math.hypot(projectionPosition,
        Game.playerDistance);
      this.steps.push(Math.asin(projectionPosition / viewDistance));
      this.dist.push(0); // * initialise distance array elements
    }
    this.halfStep = (Game.fov / this.sectionCount)*0.51;
    this.dist.push(0); // one extra element for max distance
  }
};